/*******************************************************************/
/*                                                                 */
/*   Program Name :  skdchk                                        */
/*                                                                 */
/*      Version:  2.11  2007-03-16                                 */
/*                                                                 */
/*      Copyright (c) 2002-2007 T.Kondo/NICT All Right Reserved    */
/*                                                                 */
/*******************************************************************/
/*

  Schedule File Read and Check  
            2002-06-02 
            by T. Kondo

    How to run
       skdchk sked_file [-NOEARLY]
         where sked_file ---- schedule file name
               -NOEARLY   --- ignore "tape early start" parameter

Ver 2.11 2007.03.16  improve in display in case of 7 stations. use
                     "scan" instead of "observation". improve in
					 station position display.  add # of scans by station
					 in final display
Ver 2.10 2006.12.08  add early parameter handling
Ver 2.01 2005.06.12  update due to update in common header name
2005-01-12  support VEX file
2003-08-08  support subnet

../sked/jd0306.skd
../sked/gg057c.vex
*/
#define SKDCHK_VERSION "Ver 2.11    2007-03-16"
#include "libk5.h"

int main(int argc, char *argv[])
{
      //char* fname="ge154.skd";
      sked_data_t skd;
      //struct schedule skd;
      double totsec, a, b, c, d;
      double totsec_by_sta[20];
      int nscan_by_sta[20];  // 2007-03-16
      int i, k, smax, imax, k1, k2;
      int vexmode;  // 0: normal schedule file    1: VEX file  2005-01-12
	  int noearly;  // 1: no use of early parameter  0: use of early parameter
	  int secadd;
      int kid;
      char stat_id[4];  // 2005-01-12
      vex_t vex;    // 2005-01-12

      if(argc == 1){
          printf("\n");
          printf("skdchk %s\n",SKDCHK_VERSION);
          printf("Usage: skdchk sked_file [-NOEARLY]\n");
          printf("\n");
          printf("         where  sked_file ---- schedule file name\n");
          printf("                               (VEX file is supported)\n");
          printf("                -NOEARLY ---- ignore EARLY start parameter\n");
		  printf("                               in SKED or VEX\n");
          exit(0);
      }

       printf("skdchk %s\n",SKDCHK_VERSION);
       strcpy(skd.skdname,argv[1]);  
       printf("Schedule file is  %s\n",skd.skdname);
	   noearly=0;
       if(argc>=3){
          if(strncmp(argv[2],"-NOEARLY",8)==0 ||
            strncmp(argv[2],"-noearly",8)==0 ) noearly=1;   // set no early flag
	   }

       /* schedule file format check original SKED or VEX or OVEX */
       vexmode=0;
       check_sked_type(skd.skdname,skd.skedtype);
       
       if((strncmp(skd.skedtype,"VEX",3)==0) ||
           (strncmp(skd.skedtype,"OVEX",4)==0)) {    /* VEX or OVEX format*/
           printf("File type is VEX\n");
          vexmode=1;
          strcpy(vex.vex_file_name,skd.skdname);
          vex.neop=0;   /* initialize for no EOP data case*/
          vex_read(&vex);
          //SKED_mon(&vex);    //debug
          //FREQ_mon(&vex);    //debug
          //CLOCK_mon(&vex);  //debug
          //STATION_mon(&vex);  //debug
          stat_id[0]='\0';
          vex2sked3(&vex, stat_id, &skd);

          /* slect station  */

          printf(" ****** STATION ID TABLE *********\n");
          for (i=0; i<skd.nstation ; i++){
                 printf("  %s --- %s\n",skd.statid[i],skd.statnam[i]);
          }
          printf(" *********************************\n");
          printf(" Enter Station ID ----> ");
          scanf("%s",&stat_id);
          printf(" Selected station_id is %s\n",stat_id);

          // setup sked data again using given stat_id
          //vex2sked3(&vex, stat_id, &skd);

          /* Pick up scans where given_station ID is included */
          pickup_skd(&skd, stat_id);

       } else {                                   /* SKED format */
          readskd(&skd);
       }
       if(vexmode==1){
           // find index corresponding to stat_id 
            // get ID number for stat_id
            kid=0;
            for(k=0; k<skd.nstation; k++){
                if(strncmp(skd.statid[k],stat_id,strlen(stat_id))==0){
                    kid=k;
                    break;
                }
            }
       }

       //readskd(&skd);

       printf("\n");
       printf("*** SCHEDULE FILE (%s) INFORMATION ***\n",skd.skdname);
       printf("Experiment code    : %s\n",skd.expcode);
       printf("Number of stations : %d\n",skd.nstation);


       for(i=0; i < skd.nstation; i++){
           //printf("  %3d %2s %10s %10f %10f %10f\n",i+1,skd.statid[i],skd.statnam[i],
           printf("  %3d %2s %10s %16f %16f %16f\n",i+1,skd.statid[i],skd.statnam[i],
                         skd.xyz[i][0],skd.xyz[i][1],skd.xyz[i][2]);
       }
       printf("Number of stars : %d\n",skd.nstar);

       if(skd.nstar > 20) {
          imax=20;
          printf("   (only 20 stars are listed here)\n");
       } else {
          imax=skd.nstar;
       }

       for(i=0; i < imax; i++)
       {
           printf("  %3d %10s %10s %10f %10f %10f\n",i+1,skd.starnam1[i],skd.starnam2[i],
                         skd.ra[i],skd.dec[i],skd.epo[i]);
       }
       if(vexmode==1){
           printf("Number of scans for %10s: %d\n",skd.statnam[kid],skd.nobs);
       } else {
           printf("Number of scans : %d\n",skd.nobs);
       }
       printf("   First 5 scans are as follows:\n");
       for (i=0; i < 5; i++)
       {
          printf("    %4d %10s %2d/%03d %02d:%02d:%02d  %3d %s\n",i+1,skd.obsstar[i],
                   skd.start[i][0],skd.start[i][1],skd.start[i][2],
                   skd.start[i][3],skd.start[i][4],skd.dura[i],skd.obsid[i]);
          //printf(" %d\n",strlen(skd.obsid[i]));
       }

       printf("   Last 5 scans are as follows:\n");
       for (i=skd.nobs-5; i < skd.nobs; i++)
       {
          printf("    %4d %10s %2d/%03d %02d:%02d:%02d  %3d %s\n",i+1,skd.obsstar[i],
                 skd.start[i][0],skd.start[i][1],skd.start[i][2],
                 skd.start[i][3],skd.start[i][4],skd.dura[i],skd.obsid[i]);
       }

       /* Disk requirement check */
       totsec=0.0;
       for(k=0; k<skd.nstation; k++) {
		   totsec_by_sta[k]=0.0;
		   nscan_by_sta[k]=0;  // 2007-03-16
	   }

	   secadd=skd.earlysec;

	   printf("\n     Early Start Parameter (sec) : %d",skd.earlysec);
	   if(noearly==1) {
		   printf("  but omitted by operator!");
		   secadd=0;
	   }
	   printf("\n");

       for(i=0; i< skd.nobs; i++){
           if(vexmode==1){
              totsec += (double)skd.datastop[i][kid]+secadd;
           } else {
              totsec += (double)skd.dura[i]+secadd;
           }
           for(k=0; k<skd.nstation; k++){
              totsec_by_sta[k]+= (double)skd.datastop[i][k]+secadd;
			  if(skd.datastop[i][k] > 0.0) nscan_by_sta[k]++;  // 2007-03-16
           }
       }
       a=totsec*32.0e6;  // total bit  in case of 32Mbps
       b=totsec*64.0e6;  // total bit  in case of 64Mbps
       c=totsec*128.0e6;  // total bit  in case of 128Mbps
       d=totsec*256.0e6;  // total bit  in case of 256Mbps
       a /= 8.0e9;   // in Gbytes
       b /= 8.0e9;   // in Gbytes
       c /= 8.0e9;   // in Gbytes
       d /= 8.0e9;   // in Gbytes
       printf("\n");
       if(vexmode==1){
           printf("    Disk requirements for %10s\n",skd.statnam[kid]);
       } else {
           printf("    Maximum Disk requirements\n");
       }
       printf("      Total observation time (sec)  = %d\n",(int)totsec);
       printf("       Disk requirements\n");
       printf("           32Mbps  : %10.3f GBytes\n",a);
       printf("           64Mbps  : %10.3f GBytes\n",b);
       printf("          128Mbps  : %10.3f GBytes\n",c);
       printf("          256Mbps  : %10.3f GBytes\n",d);

       if(vexmode==1) goto bypass;
       smax=7;
	   k1=0; 
	   while(1){
          k2=k1+smax;
		  if(k2 > skd.nstation) k2=skd.nstation;
          printf("\n");
          printf("                   Disk Requirements by Station (GBytes)\n");
          for(k=0; k<78; k++) printf("="); printf("\n");
          printf("        ");
          for(k=k1; k<k2; k++) printf("%10s",skd.statnam[k]); printf("\n");
          for(k=0; k<78; k++) printf("-"); printf("\n");
          printf("    sec ");
          for(k=k1; k<k2; k++) printf("%10d",(int)totsec_by_sta[k]); printf("\n");
          printf(" #scans ");
          for(k=k1; k<k2; k++) printf("%10d",nscan_by_sta[k]); printf("\n");
          for(k=0; k<78; k++) printf("-"); printf("\n");
          printf(" 32Mbps ");
          for(k=k1; k<k2; k++) printf("%10.1f",totsec_by_sta[k]*4e-3); printf("\n");
          printf(" 64Mbps ");
          for(k=k1; k<k2; k++) printf("%10.1f",totsec_by_sta[k]*8e-3); printf("\n");
          printf("128Mbps ");
          for(k=k1; k<k2; k++) printf("%10.1f",totsec_by_sta[k]*16e-3); printf("\n");
          printf("256Mbps ");
          for(k=k1; k<k2; k++) printf("%10.1f",totsec_by_sta[k]*32e-3); printf("\n");
          for(k=0; k<78; k++) printf("="); printf("\n");
		  k1+=smax;
		  //if(k1 >= skd.nstation) break;
		  if(k1 >= skd.nstation) break;  // 2008-03-16
	   }

bypass:

       if(!system(NULL)) {  // command processor not implemwnted
          return 0;
       }
#ifndef WIN32
       /* Display Disk avilable size */
       printf("\n");
       printf("Disk mounted on your system is as follows:\n");
       printf("\n");
       system("df");
       printf("\n");
#endif

    return 0;
}
